#! /usr/bin/env python

import os, sys
import pygame
import basicSprite
try:
	import cPickle
except:
	print "cpickle needed to save levels...but couldn't import it???"
	sys.exit(1)
from pygame.locals import *
from helpers import *


if not pygame.font: print 'Warning, fonts disabled'
if not pygame.mixer: print 'Warning, sound disabled'

BLOCK_SIZE = 24

class TileSprite(basicSprite.Sprite):
    """This is our sMonster that will move around the screen"""

    def __init__(self, centerPoint, image, image_file):

        basicSprite.Sprite.__init__(self, centerPoint, image)
        self.image_file = image_file

class SLEditor:
	"""Simple Level Editor"""

	def __init__(self, image_dir, width=640,height=480, level_file=""):
		"""Initialize"""
		"""Initialize PyGame"""
		pygame.init()


		self.tile_images = []
		self.read_images(image_dir)

		self.num_vertical = height/BLOCK_SIZE
		# Figure out how many extra rows we need for the images
		extra_width = (len(self.tile_images)/self.num_vertical)
		if ((len(self.tile_images)%self.num_vertical)>0):
			extra_width += 1
		# Not get teh extra width
		extra_width = extra_width * BLOCK_SIZE
		"""Set the window Size"""
		self.board_width = width
		self.height = height
		self.width = width + extra_width
		"""Create the Screen"""
		self.screen = pygame.display.set_mode((self.width, self.height))
		self.board_rect = pygame.Rect((0, 0),(self.board_width, self.height))
		self.load_images()

		# Cursor sprite
		# Cursor block highlight
		self.cursor_image, rect = load_image("cursor_rect.png",(255, 0, 255, 255))
		self.cursor_sprite = basicSprite.Sprite(((BLOCK_SIZE/2),(BLOCK_SIZE/2))
										, self.cursor_image)
		self.cursor_sprites = pygame.sprite.RenderUpdates(self.cursor_sprite)
		# Selection
		self.selection_sprite = None
		self.selected_tile = None
		self.selection_sprites = pygame.sprite.RenderUpdates()
		# Level
		self.level_sprites = pygame.sprite.OrderedUpdates()
		# Load the level that they passed in
		self.level_file = level_file;
		self.load_level(level_file)

		# By default no drag rect
		self.drag_rect = None

	def read_images(self, image_dir):
		"""Load all of the images from a directory, these will be the tiles.
		@param image_dir - string
		"""

		for root, dirs, files in os.walk(image_dir):
			for file in files:
				file_name,ext = os.path.splitext(file)
				if (ext==".png"):
					full_path = os.path.join(root, file)
					if os.path.isfile(full_path):
						self.tile_images.append(full_path)

	def load_image(self, full_name, colorkey=None):

		try:
			image = pygame.image.load(full_name)
		except pygame.error, message:
			print 'Cannot load image:', full_name
			raise SystemExit, message
		image = image.convert()
		if colorkey is not None:
			if colorkey is -1:
				colorkey = image.get_at((0,0))
			image.set_colorkey(colorkey, RLEACCEL)
		return image

	def load_images(self):
		"""Load the images from our tile images into sprites"""
		rect = pygame.Rect((self.board_width,0),(BLOCK_SIZE,BLOCK_SIZE))
		self.tile_sprites = pygame.sprite.RenderUpdates()

		count = 0

		for image_file in self.tile_images:
			self.tile_sprites.add(TileSprite(rect.center, self.load_image(image_file, -1),image_file))
			count += 1
			if (count >= self.num_vertical):
				rect.move_ip(BLOCK_SIZE,0)
				rect.top = 0
				count = 0
			else:
				rect.move_ip(0,BLOCK_SIZE)

	def MainLoop(self):
		"""This is the Main Loop of the Game"""


		"""Create the background"""
		self.background = pygame.Surface(self.screen.get_size())
		self.background = self.background.convert()
		self.background.fill((124,124,124))
		pygame.display.update()
		"""Draw the blocks onto the background, since they only need to be
		drawn once"""
		self.tile_sprites.draw(self.background)

		self.screen.blit(self.background, (0, 0))

		#self.pellet_sprites.draw(self.screen)
		#self.pellet_sprites.draw(self.background)

		pygame.display.flip()
		while 1:

			for event in pygame.event.get():
				if event.type == pygame.QUIT:
					sys.exit()
				elif event.type == MOUSEMOTION:
					self.on_mouse_move(event)
				elif event.type == MOUSEBUTTONDOWN:
					self.on_mouse_button_down(event)
				elif event.type == KEYDOWN:
					if (event.key == 115):
						# The pressed S time to save!
						self.save_level(self.level_file)

			"""Do the Drawging"""
			self.screen.blit(self.background, (0, 0))

			reclist = []
			reclist += self.cursor_sprites.draw(self.screen)
			reclist += self.selection_sprites.draw(self.screen)
			reclist += self.level_sprites.draw(self.screen)

			pygame.display.update(reclist)
			# pygame.display.flip()

	def save_level(self, level_file):

		level_list = []
		image_dict = {}
		image_count = 0

		rect = pygame.Rect((0,0),(BLOCK_SIZE,BLOCK_SIZE))
		# dummpy sprite for collision
		dummy_sprite = basicSprite.Sprite(rect.center, self.cursor_image)
		for y in range(0,self.height/BLOCK_SIZE):
			row = []
			for x in range(0,self.board_width/BLOCK_SIZE):
				current_sprites = []
				#Move the rect into the current position
				dummy_sprite.rect.left = x * BLOCK_SIZE
				dummy_sprite.rect.top = y * BLOCK_SIZE
				lst_sprites = pygame.sprite.spritecollide(dummy_sprite
											, self.level_sprites
											, False)
				if (len(lst_sprites)>0):
					for sprite in lst_sprites:
						# Go through the sprites and add them to a list
						# that will be added to the levl list
						file_name = os.path.basename(sprite.image_file)
						if (not image_dict.has_key(file_name)):
							# Add sprite to dict
							image_dict[file_name] = image_count
							image_count += 1
						current_sprites.append(file_name)

				row.append(current_sprites)

			# Now add the row to the level list
			level_list.append(row)
		try:
			save_file = open(level_file, 'wb')
			cPickle.dump(image_dict, save_file)
			cPickle.dump(level_list, save_file)
			save_file.close()
		except:
			print "Error saving level???"

	def load_level(self, level_file):
		"""Load a level
		@param level_file - string - fullpath tp level file
		to open
		"""
		# Load from file
		try:
			file = open(level_file, 'rb')
			image_dict = cPickle.load(file)
			level_list = cPickle.load(file)
			file.close()
		except:
			#failed just get outta here maybe the file doesn't exist
			print "Failed to open file"
			return

		# Now we need to populate the dictionary with the right image
		for sprite in self.tile_sprites:
			""" For each sprite in the group get the file
			name and see if it's in the dict"""
			file_name = os.path.basename(sprite.image_file)
			if (image_dict.has_key(file_name)):
				"""Alright this is valid and is in the level so
				set the data to be the image so that we can use it
				later on"""
				sprit_info = [sprite.image, sprite.image_file]
				image_dict[file_name] = sprit_info

		# Now we need to load the fle
		rect = pygame.Rect((0,0),(BLOCK_SIZE,BLOCK_SIZE))
		x = 0;
		y = 0;
		for row in level_list:
			for item_list in row:
				rect.left = x * BLOCK_SIZE
				rect.top = y * BLOCK_SIZE
				for tile in item_list:
					self.level_sprites.add(TileSprite(rect.center
						, image_dict[tile][0], image_dict[tile][1]))

				x += 1
			y += 1
			x = 0






	def get_mouse_block_rect(self):
		"""This function is to get the block rectangle
		associated with the current mouse position"""

		x_pos, y_pos = pygame.mouse.get_pos()
		x_pos = ((x_pos / BLOCK_SIZE)) * BLOCK_SIZE
		y_pos = ((y_pos / BLOCK_SIZE)) * BLOCK_SIZE

		return pygame.Rect((x_pos, y_pos), (BLOCK_SIZE,BLOCK_SIZE))

	def on_mouse_move(self, event):
		"""Called when the event loop gets a mouse down event
		@parm event - pygame.event.Event - MOUSEMOTION	     pos, rel, buttons
		"""
		#print event.pos
		#print event.rel
		#print event.buttons

		if (event.buttons[0]==0):
			# No button down
			self.cursor_sprite.rect = self.get_mouse_block_rect()
			self.drag_rect = None
		elif (self.selected_tile!=None):
			# They are left button dragging and something is selected
			cursor_rect = self.get_mouse_block_rect()
			if (self.drag_rect):
				"""We have a drag rect, so check to make sure that
				it is different then the current rect, and within the
				board boundry. If so then add the iamge"""
				if ((self.drag_rect!=cursor_rect)
					and (self.board_rect.contains(self.drag_rect))):
					# Allright add
					self.add_level_sprite(cursor_rect, self.selected_tile.image
						, self.selected_tile.image_file)

			self.drag_rect = cursor_rect
			self.cursor_sprite.rect = cursor_rect

	def add_level_sprite(self, rect, image, image_file):
		"""Add a level sprite to the level
		@param rect - rect to use
		@param image - image for the sprite
		@param image_file - full path to the image
		"""

		image_sprite = TileSprite(rect.center, image, image_file)

		""" Don't add duplicate sprites, so check collisions before
		adding
		"""
		lst_sprites = pygame.sprite.spritecollide(image_sprite
											, self.level_sprites
											, False)
		add_sprite = True
		for sprite in lst_sprites:
			if (sprite.image_file == image_file):
				add_sprite = False

		# Can we add?
		if (add_sprite):
			self.level_sprites.add(image_sprite)


	def on_mouse_button_down(self, event):
		"""Called when the event loop gets a mouse down event
		@parm event - pygame.event.Event - MOUSEBUTTONDOWN  pos, button
		"""

		if (event.button==3):
			# Right click so remove items
			pygame.sprite.spritecollide(self.cursor_sprite
											, self.level_sprites
											, True)
		else:
			# Add
			lst_tiles = pygame.sprite.spritecollide(self.cursor_sprite
												, self.tile_sprites
												, False)

			if (len(lst_tiles)>0):
				self.selected_tile = lst_tiles[0]
				# We have clicked on a sprite
				if (not self.selection_sprite):
					self.selection_sprite = basicSprite.Sprite(self.cursor_sprite.rect.center
							, self.cursor_image)
					self.selection_sprites.add(self.selection_sprite)
				else:
					self.selection_sprite.rect = self.cursor_sprite.rect
			elif (self.selected_tile!=None):
				# we have a selected tile so lets add it to the screen
				self.add_level_sprite(self.cursor_sprite.rect
							, self.selected_tile.image, self.selected_tile.image_file)
				# save for the drag
				self.drag_rect = self.cursor_sprite.rect

		"""if (self.is_rect_safe_for_hole(self.cursor_sprite.rect)):
			# Add a hole down
			if pygame.key.get_mods() & KMOD_CTRL:
				if (self.hole_out_sprite):
					self.hole_out_rect = self.hole_out_sprite.rect = self.cursor_sprite.rect
				else:
					self.hole_out_sprite = basicSprite.Sprite(self.cursor_sprite.rect.center
						, self.hole_out_image)
					self.hole_out_sprites.add(self.hole_out_sprite)
					self.hole_out_rect = self.cursor_sprite.rect
			else:
				self.hole_in_sprites.add(basicSprite.Sprite(self.cursor_sprite.rect.center
					, self.hole_in_image))"""




if __name__ == "__main__":
    MainWindow = SLEditor("/home/selsine/Projects/PyWeek/data/images/", 792,600
		, "/home/selsine/Projects/PyWeek/level_001.sle")
    MainWindow.MainLoop()
