#! /usr/bin/env python
import pygame
import random
import movingSprite
from helpers import *

RABBIT_EATING_START = pygame.USEREVENT + 1
RABBIT_EATING_OVER = pygame.USEREVENT + 2


class Bunny(movingSprite.Sprite):
	"""This is our bunny that will move around the screen"""

	def __init__(self, centerPoint, image_list, fps=150):

		movingSprite.Sprite.__init__(self, centerPoint, image_list, fps, 6)

		# Not Eating
		self.eating = False
		# Eaten
		self.eaten = 0
		self.eat_end_time = 0

	def update(self, ticks, block_group, hole_in_group, hole_out_rect
				, carrot_group):
		"""Called when the Snake sprit should update itself
		@param ticks - pygame.time.get_ticks() used to update
		the animation
		@param pygame.sprite.RenderUpdates() - block_group - The block
		sprites, used for collision detection
		@param pygame.sprite.RenderUpdates() - hole_out_group - A group
		representing down holes
		@param pygame.Rect - hole_out_rect - A rect representing
		the hole up. (may be None)
		@param pygame.sprite.RenderUpdates() - carrot_group Carrots!"""

		force_move = False

		if (self.eating):
			if (ticks>=self.eat_end_time):
				# We are done eating
				self.eating = False
				self.eat_end_time = 0
				# Get rid of the carrot we hit
				list_carrots = pygame.sprite.spritecollide(self, carrot_group, True)
				if (len(list_carrots)>0):
					# Tell the main that we are done
					pygame.event.post(pygame.event.Event(RABBIT_EATING_OVER, {}))
				# Force a move already!
				force_move = True

		if ((((ticks - self.last_update) > self.delay) and (not self.eating))
				or (force_move)):
			self.frame += 1
			if self.frame >= len(self.image_list):
				self.frame = 0
			# now Move
			xMove,yMove = self.update_move(self.frame)

			self.last_update = ticks

			if ((pygame.sprite.spritecollideany(self, hole_in_group)) and (hole_out_rect)):
				# Fell down a rabbit hole
				self.rect = pygame.Rect(hole_out_rect)
				self.direction = random.randint(1,4)
			elif pygame.sprite.spritecollideany(self, block_group):
				"""IF we hit a block, don't move - reverse the movement"""
				self.rect.move_ip(-xMove,-yMove)
				self.direction = random.randint(1,4)
			elif (pygame.sprite.spritecollideany(self, carrot_group)):
				# Time to start eating
				pygame.event.post(pygame.event.Event(RABBIT_EATING_START,{"SPRIT" : self}))
				self.eating = True
				self.eat_end_time = pygame.time.get_ticks() + 3000



