#! /usr/bin/env python
import pygame
import random
from helpers import *



class Sprite(pygame.sprite.Sprite):
	"""This is our bunny that will move around the screen"""

	def __init__(self, centerPoint, image_list, fps=150, dist = 3):

		pygame.sprite.Sprite.__init__(self)
		"""Set the image and the rect"""
		self.image_list = image_list
		self.rect = image_list[0].get_rect()
		self.image = image_list[0]
		"""Move the rect into the correct position"""
		self.rect.center = centerPoint

		# Animation
		self.start = pygame.time.get_ticks()
		self.delay = 1000 / fps
		self.last_update = 0
		self.frame = 0
		#self.update(pygame.time.get_ticks())

		# Movement
		self.direction = random.randint(1,4)
		self.dist = dist


	def MoveKeyDown(self, key):
		"""This function sets the xMove or yMove variables that will
		then move the snake when update() function is called.  The
		xMove and yMove values will be returned to normal when this
		keys MoveKeyUp function is called."""

		if (key == K_RIGHT):
			self.xMove += self.x_dist
		elif (key == K_LEFT):
			self.xMove += -self.x_dist
		elif (key == K_UP):
			self.yMove += -self.y_dist
		elif (key == K_DOWN):
			self.yMove += self.y_dist

	def MoveKeyUp(self, key):
		"""This function resets the xMove or yMove variables that will
		then move the snake when update() function is called.  The
		xMove and yMove values will be returned to normal when this
		keys MoveKeyUp function is called."""

		if (key == K_RIGHT):
			self.xMove += -self.x_dist
		elif (key == K_LEFT):
			self.xMove += self.x_dist
		elif (key == K_UP):
			self.yMove += self.y_dist
		elif (key == K_DOWN):
			self.yMove += -self.y_dist

	def move_mouse(self):
		"""Mouse with the mouse"""
		pos = pygame.mouse.get_pos()
		self.rect.midtop = pos

	def update_move(self, frame):
		"""Called when the moving sprite shoudl update itself due to
		Update being called.  This function will update the position
		of the sprite and load the correct image
		@param frame - integer representing the current frame, probably
		just self.frame, but you could pass something extra if you
		wanted to.
		@param return xMove,yMove integer values, the direction that
		the sprite was moved in.  Returned so that in situations of
		collision detection you could reverse the movement.
		"""

		xMove,yMove = 0,0
		rotate = pygame.transform.rotate
		if self.direction==1:
			# Left
			self.image = rotate(self.image_list[frame], 90)
			xMove = -self.dist
		elif self.direction==2:
			# Up
			self.image = self.image_list[self.frame]
			yMove = -self.dist
		elif self.direction==3:
			# Right
			self.image = rotate(self.image_list[frame], 270)
			xMove = self.dist
		elif self.direction==4:
			# down
			self.image = rotate(self.image_list[frame],180)
			yMove = self.dist

		self.rect.move_ip(xMove,yMove) #Move the Rect

		return xMove,yMove

	def update(self, ticks, block_group):

		if ((ticks - self.last_update) > self.delay):
			self.frame += 1
			if self.frame >= len(self.image_list):
				self.frame = 0
			# now Move
			xMove,yMove = self.update_move(self.frame)

			self.last_update = ticks

			if pygame.sprite.spritecollideany(self, block_group):
				"""IF we hit a block, don't move - reverse the movement"""
				self.rect.move_ip(-xMove,-yMove)
				self.direction = random.randint(1,4)


